'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var uuid = require('uuid');
var _ = _interopDefault(require('lodash'));

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

var createReducer = function createReducer() {
  return function reducer(state, action) {
    var _extends2;

    switch (action.type) {
      case "insert":
        var id = uuid.v4();

        var data = _extends({}, state.data, (_extends2 = {}, _extends2[id] = {
          id: id,
          children: [],
          parentId: action.parentId,
          data: _extends({}, action.item, {
            id: id
          })
        }, _extends2));

        if (action.parentId) {
          var _data$action$parentId;

          data[action.parentId] = _extends({}, data[action.parentId], {
            children: [].concat((_data$action$parentId = data[action.parentId]) == null ? void 0 : _data$action$parentId.children, [id])
          });
        }

        return _extends({}, state, {
          root: _(data).values().filter(function (i) {
            return i && !i.parentId;
          }).map(function (i) {
            return i.id;
          }).valueOf(),
          data: data
        });

      case "push":
        var p = state.path;

        if (action.section !== undefined && p[action.section]) {
          p.splice(action.section, 1, action.item);
          p.splice(action.section + 1, p.length - action.section);
        } else {
          p.push(action.item);
        }

        return _extends({}, state, {
          path: [].concat(p)
        });

      case "pop":
        state.path.pop();
        return _extends({}, state, {
          path: [].concat(state.path)
        });

      default:
        return state;
    }
  };
};

var getItem = function getItem(id, data) {
  var _data$id;

  return data == null ? void 0 : (_data$id = data[id]) == null ? void 0 : _data$id.data;
};

var getChildren = function getChildren(id, context) {
  var _context$data, _context$data$id;

  return createItemsProps(((_context$data = context.data) == null ? void 0 : (_context$data$id = _context$data[id]) == null ? void 0 : _context$data$id.children) || [], context);
};

var createItemProps = function createItemProps(id, context) {
  var data = context.data,
      push = context.push,
      path = context.path;
  var item = getItem(id, data);
  Object.assign(item, {
    //data: () => item,
    isSelected: path.includes(id),
    children: function children() {
      return getChildren(id, context);
    },
    pushAt: function pushAt(atSection) {
      return push(id, atSection);
    },
    buildProps: function buildProps(additional) {
      return _extends({}, additional, {
        key: id
      });
    }
  });
  return item; // return {
  //     //data: () => item,
  //     isSelected: path.includes(id),
  //     children: () => getChildren(id, context),
  //     pushAt: (atSection: number) => push(id, atSection),
  //     buildProps: (additional?: object) => ({
  //         ...additional,
  //         key: id
  //     })
  // };
};
var createItemsProps = function createItemsProps(ids, context) {
  return {
    original: ids,
    length: ids.length,
    includes: ids.includes,
    map: function map(callbackFn) {
      return ids.map(function (id, index) {
        return callbackFn(createItemProps(id, context), index, ids);
      });
    },
    forEach: function forEach(callbackFn) {
      return ids.forEach(function (id, index) {
        return callbackFn(createItemProps(id, context), index, ids);
      });
    }
  };
};
var buildOptions = function buildOptions(options) {
  if (!options) {
    return undefined;
  }

  var data = {}; // TODO
  // options.initialValues?.forEach((item) => {
  //     const id = uuid();
  //     data[item] = {
  //         id,
  //         children: [],
  //         parentId: action.parentId,
  //         data: _.omit(action.item, "id") as T
  //     };
  // });

  return {
    path: options.path || [],
    root: [],
    data: data
  };
};

/**
 * React Column View
 * Riccardo Caranfil
 *
 * Created by Riccardo Caranfil on 05/05/21
 */
var INITIAL_STATE = {
  path: [],
  root: [],
  data: {}
};
var useColumnView = function useColumnView(options) {
  var stateReducer = createReducer();

  var _useReducer = react.useReducer(stateReducer, buildOptions(options) || INITIAL_STATE),
      _useReducer$ = _useReducer[0],
      root = _useReducer$.root,
      data = _useReducer$.data,
      path = _useReducer$.path,
      dispatch = _useReducer[1];

  var insert = react.useCallback(function (item, parentId) {
    return dispatch({
      type: "insert",
      item: item,
      parentId: parentId
    });
  }, [dispatch]);
  var push = react.useCallback(function (item, section) {
    return dispatch({
      type: "push",
      item: item,
      section: section
    });
  }, [dispatch]); // const pop = useCallback((item: string) => dispatch({type: "pop", item}), [dispatch]);

  return {
    root: createItemsProps(root, {
      push: push,
      path: path,
      data: data
    }),
    path: createItemsProps(path, {
      push: push,
      path: path,
      data: data
    }),
    insert: insert
  };
};

exports.useColumnView = useColumnView;
//# sourceMappingURL=react-column-view.cjs.development.js.map
